<?php

namespace App\Http\Controllers\Api;


use App\Http\Controllers\Controller;
use App\Http\Requests\StoreStationRequest;
use App\Http\Requests\UpdateStationRequest;
use App\Models\Station;
use Illuminate\Http\Resources\Json\JsonResource;
use Spatie\RouteAttributes\Attributes\Resource;
use Illuminate\Database\Eloquent\Builder;

#[Resource(
    resource: 'stations',
    apiResource: true,
    only: ['index', 'update']
)]
class StationController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $data = Station::with('channels')->orderBy('name')->when(request('query', ''), function (Builder $query, string $search) {
            $query->where('name', 'LIKE', '%' . $search . '%')
                ->orWhere('sn', 'LIKE', '%' . $search . '%')
                ->orWhere('code', 'LIKE', '%' . $search . '%');
        })->paginate(request('limit', 20));

        return JsonResource::collection($data);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreStationRequest $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(Station $station)
    {
        return new JsonResource($station);
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Station $station)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(UpdateStationRequest $request, Station $station)
    {
        $station->fill($request->all());
        $station->save();

        $data = collect($request->channels)
            ->map(fn ($c) => $c['pivot'])
            ->keyBy('channel_id')
            ->map(fn ($c) => ['properties' => $c['properties']]);

        $station->channels()->sync($data);

        return $this->show($station->load('channels'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Station $station)
    {
        //
    }
}
