<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\StoreWeatherDataRequest;
use App\Http\Requests\UpdateWeatherDataRequest;
use App\Http\Resources\WeatherData as ResourcesWeatherData;
use App\Models\WeatherData;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Facades\DB;
use Spatie\RouteAttributes\Attributes\Resource;
use Illuminate\Database\Eloquent\Builder;

#[Resource(
    resource: 'weather_data',
    apiResource: true,
    only: ['index']
)]
class WeatherDataController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        // \DB::raw('GROUP_CONCAT( DISTINCT CONCAT(software_channel_name,\':\',software_channel_value)  order by software_channel_name SEPARATOR \';\' ) as \'values\''),
        $data = WeatherData::with('station')
            ->select('software_channel_sn', 'software_channel_datetime',
                DB::raw('CONCAT("[",GROUP_CONCAT( JSON_OBJECT("id",id,"name",software_channel_name,"value",CAST(software_channel_value as CHAR))  order by software_channel_name ), "]") as \'data\''),
            )
            ->when(request('station', ''), function (Builder $query, string $station) {
                return $query->where('software_channel_sn', $station);
            })->when(request('date_from', ''), function (Builder $query, string $date_from) {
                return $query->where('software_channel_datetime','>=', $date_from);
            })->when(request('date_to', ''), function (Builder $query, string $date_to) {
                return $query->where('software_channel_datetime','<=', $date_to);
            })->when(request('channel', ''), function (Builder $query, string $channel) {
                return $query->where('software_channel_name', $channel);
            })
            ->groupBy(['software_channel_sn', 'software_channel_datetime'])
            ->orderBy('software_channel_datetime', 'DESC')
            ->paginate(request('limit',20));

        return ResourcesWeatherData::collection($data);

    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreWeatherDataRequest $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(WeatherData $weatherData)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(WeatherData $weatherData)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(UpdateWeatherDataRequest $request, WeatherData $weatherData)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(WeatherData $weatherData)
    {
        //
    }
}
