<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;

class ModxController extends Controller
{

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->addModxContext();
            return $next($request);
        });
    }

    public function connectors($path = null)
    {
        require_once MODX_CONNECTORS_PATH . $path;
        if ($path === 'index.php') $modx->request->handleRequest();
    }

    public function manager($path = null)
    {
        if ($path === null) {

            require_once MODX_CORE_PATH . 'vendor/autoload.php';

            /* @var \MODX\Revolution\modX $modx create the modX object */
            $modx = new \MODX\Revolution\modX('', [\xPDO\xPDO::OPT_CONN_INIT => [\xPDO\xPDO::OPT_CONN_MUTABLE => true]]);
            $modx->initialize('mgr');
            $modx->getRequest();
            $modx->getParser();

            $modx->request->handleRequest();
        } else {
            $path = explode('?', $path);
            $path = reset($path);
            $path = base_path("modx/manager/$path");

            if (!File::exists($path)) abort(404);

            $mimeTypes = new \Symfony\Component\Mime\MimeTypes();
            $ContentType =  match (true) {
                str_ends_with($path, '.js') => 'application/javascript',
                str_ends_with($path, '.css') => 'text/css',
                default => $mimeTypes->guessMimeType($path)
            };

            return response()->file($path, [
                'Content-Type' => $ContentType
            ]);
        }
        @session_write_close();
        exit();
    }

    public function assets($path = null)
    {
        try {
            return response()->file(base_path("modx/assets/$path"));
        } catch (Throwable $th) {
            abort(404);
        }
    }

    public function index($path = null)
    {
        require_once MODX_CORE_PATH . 'vendor/autoload.php';

        /* start output buffering */
        ob_start();

        global $modx;
        /* Create an instance of the modX class */
        $modx = new \MODX\Revolution\modX();
        /* Set the actual start time */
        $modx->startTime = microtime(true);
        $modx->initialize('web');
        $modx->handleRequest();

        @session_write_close();
        exit();
    }

    protected function addModxContext(): void
    {
        define('MODX_CONFIG_KEY', 'config');
        define('MODX_CORE_PATH', base_path('modx/core/'));

        $config_path = MODX_CORE_PATH . 'config/' . MODX_CONFIG_KEY . '.inc.php';
        if (!File::exists($config_path)) copy(MODX_CORE_PATH . 'config/tpl.inc.php', $config_path);

        require_once $config_path;
    }
}
