<?php

namespace App\Services;

use App\Models\File;
use App\Models\Station;
use App\Models\WeatherData;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Storage;
use \Illuminate\Support\Str;

class ParseFile
{
    public function process(File $file)
    {
        $properties = $file->properties;
        $properties['start'] = time();
        $count = 0;
        $skipLines = true;
        $parts = [];

        $text = Storage::disk($file->disk ?? 'local')->get($file->stub);

        $sn = Str::match("/Serial Number: ([\w\d]+)/i", $text);
        $station = Station::with('channels')->whereSn($sn)->firstOrFail();

        $lines = preg_split("/\r?\n|\r/", $text);
        try {
            foreach ($lines as $line) {
                if ($skipLines) {
                    if (Str::startsWith($line, 'Date')) {
                        $parts = $this->parse($line);
                        $skipLines = false;
                    }
                    continue;
                } 

                $data = $this->parse($line);
                foreach ($data as $key => $val) {
                    if ($key < 2) continue;
                    $channel = $station->channels->first(fn ($v) => in_array($parts[$key], $v->pivot->properties->names));
                    if(empty($channel)) continue;
                    // \Log::info( $key .  ($channel ?? 'N/A') );
                    WeatherData::firstOrCreate([
                        'software_channel_sn' => $station->sn,
                        'software_channel_datetime' => Carbon::createFromFormat('d/m/Y H:i:s', "$data[0] $data[1]"),
                        'software_channel_name' => $channel->id,
                    ], [
                        'software_channel_value' => $val
                    ]);
                }
                $count++;
                
            }
            $properties['lines'] = $count;
            $properties['status'] = 'success';
        }catch(\Exception $e) {
            $properties['status'] = 'fail';
        }
        $properties['end'] = time();
        $file->properties = $properties;
        $file->save();
    }

    protected function parse($str)
    {
        return array_values(array_filter(str_getcsv($str, ' ')));
    }
}
