import { createWebHistory, createRouter } from "vue-router";
import { useUserStore } from "@/store";
import { storeToRefs } from "pinia";

/* Guest Component */
const Login = () => import("@/pages/Login.vue");
// /* Guest Component */

// /* Layouts */
const Layout = () => import("@/layouts/Default.vue");
// /* Layouts */

// /* Authenticated Component */
const Dashboard = () => import("@/pages/Dashboard.vue");
const Stations = () => import("@/pages/Stations.vue");
const WeatherData = () => import("@/pages/WeatherData.vue");
const Files = () => import("@/pages/Files.vue");
/* Authenticated Component */

const routes = [
    {
        name: "Login",
        path: "/login",
        component: Login,
        meta: {
            middleware: "guest",
            title: `Login`,
        },
    },
    {
        path: "/",
        component: Layout,
        children: [
            {
                name: "Dashboard",
                path: "/",
                component: Dashboard,
                meta: {
                    title: `Dashboard`,
                },
            },
            {
                name: "Stations",
                path: "/stations",
                component: Stations,
                meta: {
                    title: `Stations`,
                },
            },
            {
                name: "WeatherData",
                path: "/weather_data",
                component: WeatherData,
                meta: {
                    title: `Weather Data`,
                },
            },
            {
                name: "Files",
                path: "/files",
                component: Files,
                meta: {
                    title: `Files`,
                },
            },
        ],
    },
    { path: '/:pathMatch(.*)*', name: "NotFound", redirect : '/' },
];

const router = createRouter({
    history: createWebHistory('admin'),
    routes, // short for `routes: routes`
});

router.beforeEach(async (to, from, next) => {
    const userStore = useUserStore();
    const { isLoggedIn } = storeToRefs(userStore);
    document.title = to.meta.title;
    await userStore.fetchUser()
    switch (to.meta.middleware) {
        case "guest":
            isLoggedIn.value ? next({ name: "Dashboard" }) : next();
            break;
        default:
            !isLoggedIn.value ? next({ name: "Login" }) : next();
            break;
    }
});

export default router;
