import { defineStore } from "pinia";

import http from '@/plugins/axios'
// You can name the return value of `defineStore()` anything you want,
// but it's best to use the name of the store and surround it with `use`
// and `Store` (e.g. `useUserStore`, `useCartStore`, `useProductStore`)
// the first argument is a unique id of the store across your application

export const useUserActions = () => {
    const userStore = useUserStore()
    const router = useRouter()

    const login = async (data) => {
        await userStore.login(data)
        router.push({ name : 'Dashboard' })
    }

    const logout = async () => {
        await userStore.logout()
        router.push({ name : 'Login' })
    }

    return {
        userStore, login, logout
    }
}

export const useUserStore = defineStore("user", {
    state: () => ({ 
        initialized: false,
        user: null,
    }),
    getters: {
        isLoggedIn: (state) => !!state.user?.id,
    },
    actions: {
        async fetchUser() {
            if(!this.initialized) {
                await http({
                    baseURL : '/',
                    method: 'GET',
                    url :'/sanctum/csrf-cookie'
                })
                this.initialized = true
            }
            if(!this.isLoggedIn){
                try {
                    const { data } = await http({
                        method: 'GET',
                        url :'/me'
                    })
                    this.user = data
                }catch(e) {}
            }
        },
        async login(data) {
            await http({
                method: 'POST',
                url : 'login',
                data
            })
            await this.fetchUser()
        },
    
        async logout() {
            await http({
                method: 'POST',
                url : 'logout'
            })
            this.user = null
        },
    },
});
